@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: actions.c 8778 2009-07-03 20:10:05Z charles $
+ * $Id: actions.c 8946 2009-08-16 03:25:58Z charles $
  */
 
 #include <string.h>
@@ -299,6 +299,15 @@ action_sensitize( const char * name,
     g_object_set( action, "sensitive", b, NULL );
 }
 
+void
+action_set_important( const char * name, gboolean b )
+{
+    GtkAction * action = get_action( name );
+
+    g_assert( action != NULL );
+    g_object_set( action, "is-important", b, NULL );
+}
+
 void
 action_toggle( const char * name,
                gboolean     b )
