@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: main.c 8766 2009-07-01 04:10:04Z charles $
+ * $Id: main.c 8929 2009-08-14 12:53:08Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -88,10 +88,6 @@ struct cbdata
     GtkTreeSelection  * sel;
 };
 
-#define CBDATA_PTR "callback-data-pointer"
-
-static GtkUIManager * myUIManager = NULL;
-
 static void           appsetup( TrWindow * wind,
                                 GSList *   args,
                                 struct     cbdata *,
@@ -101,7 +97,7 @@ static void           appsetup( TrWindow * wind,
 static void           winsetup( struct cbdata * cbdata,
                                 TrWindow *      wind );
 
-static void           wannaquit( void * vdata );
+static void           wannaquit( gpointer vdata );
 
 static void           setupdrag( GtkWidget *    widget,
                                  struct cbdata *data );
@@ -180,6 +176,7 @@ refreshActions( struct cbdata * data )
     action_sensitize( "remove-torrent", counts.totalCount != 0 );
     action_sensitize( "delete-torrent", counts.totalCount != 0 );
     action_sensitize( "verify-torrent", counts.totalCount != 0 );
+    action_sensitize( "show-torrent-properties", counts.totalCount != 0 );
     action_sensitize( "open-torrent-folder", counts.totalCount == 1 );
     action_sensitize( "relocate-torrent", counts.totalCount == 1 );
 
@@ -328,20 +325,19 @@ onRPCChanged( tr_session            * session UNUSED,
 }
 
 int
-main( int     argc,
-      char ** argv )
+main( int argc, char ** argv )
 {
-    char *              err = NULL;
-    GSList *            argfiles;
-    GError *            gerr;
-    gboolean            didinit = FALSE;
-    gboolean            didlock = FALSE;
-    gboolean            showversion = FALSE;
-    gboolean            startpaused = FALSE;
-    gboolean            startminimized = FALSE;
-    const char *        domain = MY_NAME;
-    char *              configDir = NULL;
-    tr_lockfile_state_t tr_state;
+    char * err = NULL;
+    GSList * argfiles;
+    GError * gerr;
+    gboolean didinit = FALSE;
+    gboolean didlock = FALSE;
+    gboolean showversion = FALSE;
+    gboolean startpaused = FALSE;
+    gboolean startminimized = FALSE;
+    const char * domain = MY_NAME;
+    char * configDir = NULL;
+    gtr_lockfile_state_t tr_state;
 
     GOptionEntry entries[] = {
         { "paused",     'p', 0, G_OPTION_ARG_NONE,
@@ -418,7 +414,7 @@ main( int     argc,
             }
         }
     }
-    else if( ( !didlock ) && ( tr_state == TR_LOCKFILE_ELOCK ) )
+    else if( ( !didlock ) && ( tr_state == GTR_LOCKFILE_ELOCK ) )
     {
         /* There's already another copy of Transmission running,
          * so tell it to present its window to the user */
@@ -433,14 +429,15 @@ main( int     argc,
 
         const char * str;
         GtkWindow * win;
+        GtkUIManager * myUIManager;
         tr_session * session;
         struct cbdata * cbdata = g_new0( struct cbdata, 1 );
 
         /* ensure the directories are created */
        if(( str = pref_string_get( PREF_KEY_DIR_WATCH )))
-           mkdir_p( str, 0777 );
+           gtr_mkdir_with_parents( str, 0777 );
        if(( str = pref_string_get( TR_PREFS_KEY_DOWNLOAD_DIR )))
-           mkdir_p( str, 0777 );
+           gtr_mkdir_with_parents( str, 0777 );
 
         /* initialize the libtransmission session */
         session = tr_sessionInit( "gtk", configDir, TRUE, pref_get_all( ) );
@@ -471,8 +468,11 @@ main( int     argc,
     }
     else if( err )
     {
-        gtk_widget_show( errmsg_full( NULL, (callbackfunc_t)gtk_main_quit,
-                                      NULL, "%s", err ) );
+        const char * primary_text = _( "Transmission cannot be started." );
+        GtkWidget * w = gtk_message_dialog_new( NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, primary_text, NULL );
+        gtk_message_dialog_format_secondary_text( GTK_MESSAGE_DIALOG( w ), "%s", err );
+        g_signal_connect( w, "response", G_CALLBACK(gtk_main_quit), NULL );
+        gtk_widget_show( w );
         g_free( err );
         gtk_main( );
     }
@@ -670,7 +670,7 @@ do_exit_cb( GtkWidget *w  UNUSED,
 }
 
 static void
-wannaquit( void * vdata )
+wannaquit( gpointer vdata )
 {
     GtkWidget *r, *p, *b, *w, *c;
     struct cbdata *cbdata = vdata;
@@ -831,7 +831,7 @@ setupdrag( GtkWidget *    widget,
                           gotdrag ), data );
 
     gtk_drag_dest_set( widget, GTK_DEST_DEFAULT_ALL, targets,
-                       ALEN( targets ), GDK_ACTION_COPY | GDK_ACTION_MOVE );
+                       G_N_ELEMENTS( targets ), GDK_ACTION_COPY | GDK_ACTION_MOVE );
 }
 
 static void
@@ -890,12 +890,12 @@ coreerr( TrCore * core UNUSED, guint code, const char * msg, struct cbdata * c )
 {
     switch( code )
     {
-        case TR_EINVALID:
+        case TR_PARSE_ERR:
             c->errqueue =
                 g_slist_append( c->errqueue, g_path_get_basename( msg ) );
             break;
 
-        case TR_EDUPLICATE:
+        case TR_PARSE_DUPLICATE:
             c->dupqueue = g_slist_append( c->dupqueue, g_strdup( msg ) );
             break;
 
@@ -903,10 +903,6 @@ coreerr( TrCore * core UNUSED, guint code, const char * msg, struct cbdata * c )
             showTorrentErrors( c );
             break;
 
-        case TR_CORE_ERR_SAVE_STATE:
-            errmsg( c->wind, "%s", msg );
-            break;
-
         default:
             g_assert_not_reached( );
             break;
