@@ -1,26 +1,14 @@
-/******************************************************************************
- * $Id: util.c 8758 2009-06-29 17:52:24Z charles $
+/*
+ * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * This file is licensed by the GPL version 2.  Works owned by the
+ * Transmission project are granted a special exemption to clause 2(b)
+ * so that the bulk of its code can remain under the MIT license.
+ * This exemption does not extend to derived works not owned by
+ * the Transmission project.
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- *****************************************************************************/
+ * $Id: util.c 8941 2009-08-15 21:14:44Z charles $
+ */
 
 #include <ctype.h> /* isxdigit() */
 #include <errno.h>
@@ -28,6 +16,10 @@
 #include <stdlib.h> /* free() */
 #include <string.h> /* strcmp() */
 
+#include <sys/types.h> /* for gtr_lockfile()'s open() */
+#include <sys/stat.h> /* for gtr_lockfile()'s open() */
+#include <fcntl.h> /* for gtr_lockfile()'s open() */
+
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h> /* g_unlink() */
@@ -49,6 +41,55 @@
 #include "tr-prefs.h"
 #include "util.h"
 
+gtr_lockfile_state_t
+gtr_lockfile( const char * filename )
+{
+    gtr_lockfile_state_t ret;
+
+#ifdef WIN32
+
+    HANDLE file = CreateFile( filename,
+                              GENERIC_READ | GENERIC_WRITE,
+                              FILE_SHARE_READ | FILE_SHARE_WRITE,
+                              NULL,
+                              OPEN_ALWAYS,
+                              FILE_ATTRIBUTE_NORMAL,
+                              NULL );
+    if( file == INVALID_HANDLE_VALUE )
+        ret = GTR_LOCKFILE_EOPEN;
+    else if( !LockFile( file, 0, 0, 1, 1 ) )
+        ret = GTR_LOCKFILE_ELOCK;
+    else
+        ret = GTR_LOCKFILE_SUCCESS;
+
+#else
+
+    int fd = open( filename, O_RDWR | O_CREAT, 0666 );
+    if( fd < 0 )
+        ret = GTR_LOCKFILE_EOPEN;
+    else {
+        struct flock lk;
+        memset( &lk, 0,  sizeof( lk ) );
+        lk.l_start = 0;
+        lk.l_len = 0;
+        lk.l_type = F_WRLCK;
+        lk.l_whence = SEEK_SET;
+        if( -1 == fcntl( fd, F_SETLK, &lk ) )
+            ret = GTR_LOCKFILE_ELOCK;
+        else
+            ret = GTR_LOCKFILE_SUCCESS;
+    }
+
+#endif
+
+    return ret;
+}
+
+/***
+****
+***/
+
+
 char*
 tr_strlratio( char * buf, double ratio, size_t buflen )
 {
@@ -209,8 +250,7 @@ gtr_localtime2( char * buf, time_t time, size_t buflen )
 }
 
 int
-mkdir_p( const char * path,
-         mode_t       mode )
+gtr_mkdir_with_parents( const char * path, int mode )
 {
 #if GLIB_CHECK_VERSION( 2, 8, 0 )
     return !g_mkdir_with_parents( path, mode );
@@ -306,60 +346,6 @@ checkfilenames( int    argc,
     return g_slist_reverse( ret );
 }
 
-static void
-onErrorResponse( GtkWidget * dialog,
-                 int resp    UNUSED,
-                 gpointer    glist )
-{
-    GSList * list = glist;
-
-    if( list )
-    {
-        callbackfunc_t func = list->data;
-        gpointer       user_data = list->next->data;
-        func( user_data );
-        g_slist_free( list );
-    }
-
-    gtk_widget_destroy( dialog );
-}
-
-static GtkWidget *
-verrmsg_full( GtkWindow *    wind,
-              callbackfunc_t func,
-              void *         data,
-              const char *   format,
-              va_list        ap )
-{
-    GtkWidget *dialog;
-    char *     msg;
-    GSList *   funcdata = NULL;
-
-    msg = g_strdup_vprintf( format, ap );
-
-    if( NULL == wind )
-        dialog = gtk_message_dialog_new(
-            NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", msg );
-    else
-        dialog = gtk_message_dialog_new(
-            wind,
-            GTK_DIALOG_MODAL |
-            GTK_DIALOG_DESTROY_WITH_PARENT,
-            GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-            "%s", msg );
-
-    if( func )
-    {
-        funcdata = g_slist_append( funcdata, (gpointer)func );
-        funcdata = g_slist_append( funcdata, data );
-    }
-    g_signal_connect( dialog, "response", G_CALLBACK(
-                          onErrorResponse ), funcdata );
-    g_free( msg );
-
-    return dialog;
-}
-
 void
 addTorrentErrorDialog( GtkWidget *  child,
                        int          err,
@@ -372,17 +358,9 @@ addTorrentErrorDialog( GtkWidget *  child,
 
     switch( err )
     {
-        case TR_EINVALID:
-            fmt = _( "The torrent file \"%s\" contains invalid data." );
-            break;
-
-        case TR_EDUPLICATE:
-            fmt = _( "The torrent file \"%s\" is already in use." ); break;
-
-        default:
-            fmt = _(
-                "The torrent file \"%s\" encountered an unknown error." );
-            break;
+        case TR_PARSE_ERR: fmt = _( "The torrent file \"%s\" contains invalid data." ); break;
+        case TR_PARSE_DUPLICATE: fmt = _( "The torrent file \"%s\" is already in use." ); break;
+        default: fmt = _( "The torrent file \"%s\" encountered an unknown error." ); break;
     }
     secondary = g_strdup_printf( fmt, filename );
     win = ( !child || GTK_IS_WINDOW( child ) )
@@ -401,47 +379,6 @@ addTorrentErrorDialog( GtkWidget *  child,
     gtk_widget_show_all( w );
     g_free( secondary );
 }
-
-void
-errmsg( GtkWindow *  wind,
-        const char * format,
-        ... )
-{
-    GtkWidget * dialog;
-    va_list     ap;
-
-    va_start( ap, format );
-    dialog = verrmsg_full( wind, NULL, NULL, format, ap );
-    va_end( ap );
-
-    if( NULL != wind && !GTK_WIDGET_MAPPED( GTK_WIDGET( wind ) ) )
-    {
-        g_signal_connect_swapped( wind, "map",
-                                  G_CALLBACK( gtk_widget_show ), dialog );
-    }
-    else
-    {
-        gtk_widget_show( dialog );
-    }
-}
-
-GtkWidget *
-errmsg_full( GtkWindow *    wind,
-             callbackfunc_t func,
-             void *         data,
-             const char *   format,
-             ... )
-{
-    GtkWidget * dialog;
-    va_list     ap;
-
-    va_start( ap, format );
-    dialog = verrmsg_full( wind, func, data, format, ap );
-    va_end( ap );
-
-    return dialog;
-}
-
 typedef void ( PopupFunc )( GtkWidget*, GdkEventButton* );
 
 /* pop up the context menu if a user right-clicks.
@@ -495,7 +432,6 @@ on_tree_view_button_released( GtkWidget *      view,
     {
         GtkTreeSelection * selection = gtk_tree_view_get_selection( tv );
         gtk_tree_selection_unselect_all ( selection );
-        return TRUE;
     }
 
     return FALSE;
