@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bencode.c 8825 2009-07-17 18:00:29Z charles $
+ * $Id: bencode.c 8903 2009-08-12 14:40:32Z charles $
  */
 
 #include <assert.h>
@@ -1301,7 +1301,7 @@ jsonRealFunc( const tr_benc * val, void * vdata )
         /* json requires a '.' decimal point regardless of locale */
         tr_strlcpy( locale, setlocale( LC_NUMERIC, NULL ), sizeof( locale ) );
         setlocale( LC_NUMERIC, "POSIX" );
-        evbuffer_add_printf( data->out, "%.4f", val->val.d );
+        evbuffer_add_printf( data->out, "%.4f", tr_truncd( val->val.d, 4 ) );
         setlocale( LC_NUMERIC, locale );
     }
 
@@ -1435,7 +1435,7 @@ tr_bencDictChild( tr_benc * dict, size_t n, const char ** key, tr_benc ** val )
     return success;
 }
 
-void 
+void
 tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
 {
     size_t i;
