@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr.h 7722 2009-01-16 17:31:10Z charles $
+ * $Id: peer-mgr.h 8695 2009-06-15 00:11:06Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -23,8 +23,15 @@
  #include <winsock2.h> /* struct in_addr */
 #endif
 
+#include "bitfield.h"
 #include "net.h"
 #include "publish.h" /* tr_publisher_tag */
+#include "utils.h"
+
+/**
+ * @addtogroup peers Peers
+ * @{
+ */
 
 struct tr_peer_stat;
 struct tr_torrent;
@@ -49,7 +56,6 @@ tr_pex;
 
 
 struct tr_bandwidth;
-struct tr_bitfield;
 struct tr_peerIo;
 struct tr_peermsgs;
 
@@ -60,6 +66,9 @@ enum
     ENCRYPTION_PREFERENCE_NO
 };
 
+/* opaque forward declaration */
+struct peer_atom;
+
 /**
  * State information about a connected peer.
  *
@@ -79,8 +88,10 @@ typedef struct tr_peer
 
     uint8_t                  encryption_preference;
     tr_port                  port;
+    tr_port                  dht_port;
     tr_address               addr;
     struct tr_peerIo       * io;
+    struct peer_atom       * atom;
 
     struct tr_bitfield     * blame;
     struct tr_bitfield     * have;
@@ -168,6 +179,8 @@ void tr_peerMgrTorrentStats( tr_torrent * tor,
 struct tr_peer_stat* tr_peerMgrPeerStats( const tr_torrent * tor,
                                           int              * setmeCount );
 
+float tr_peerMgrGetWebseedSpeed( const tr_torrent * tor, uint64_t now );
+
 float* tr_peerMgrWebSpeeds( const tr_torrent * tor );
 
 
@@ -175,4 +188,6 @@ double tr_peerGetPieceSpeed( const tr_peer    * peer,
                              uint64_t           now,
                              tr_direction       direction );
 
+/* @} */
+
 #endif
