@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: resume.c 8767 2009-07-01 14:58:57Z charles $
+ * $Id: resume.c 8930 2009-08-14 14:41:59Z charles $
  */
 
 #include <unistd.h> /* unlink */
@@ -76,19 +76,18 @@ static void
 savePeers( tr_benc *          dict,
            const tr_torrent * tor )
 {
-    tr_pex * pex = NULL;
-    int count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET );
+    int count;
+    tr_pex * pex;
 
+    count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET );
     if( count > 0 )
         tr_bencDictAddRaw( dict, KEY_PEERS, pex, sizeof( tr_pex ) * count );
-
     tr_free( pex );
-    pex = NULL;
-    
+
     count = tr_peerMgrGetPeers( (tr_torrent*) tor, &pex, TR_AF_INET6 );
     if( count > 0 )
         tr_bencDictAddRaw( dict, KEY_PEERS6, pex, sizeof( tr_pex ) * count );
-    
+
     tr_free( pex );
 }
 
@@ -113,7 +112,7 @@ loadPeers( tr_benc *    dict,
         tr_tordbg( tor, "Loaded %d IPv4 peers from resume file", count );
         ret = TR_FR_PEERS;
     }
-    
+
     if( tr_bencDictFindRaw( dict, KEY_PEERS6, &str, &len ) )
     {
         int       i;
@@ -481,9 +480,11 @@ tr_torrentSaveResume( const tr_torrent * tor )
     tr_benc top;
     char *  filename;
 
-    if( !tor )
+    if( !tr_isTorrent( tor ) )
         return;
 
+    tr_tordbg( tor, "Saving .resume file for \"%s\"", tor->info.name );
+
     tr_bencInitDict( &top, 32 ); /* arbitrary "big enough" number */
     tr_bencDictAddInt( &top, KEY_ACTIVITY_DATE,
                        tor->activityDate );
@@ -522,12 +523,13 @@ static uint64_t
 loadFromFile( tr_torrent * tor,
               uint64_t     fieldsToLoad )
 {
-    int64_t      i;
+    int64_t  i;
     const char * str;
-    uint64_t     fieldsLoaded = 0;
-    char *       filename;
-    tr_benc      top;
-    tr_bool      boolVal;
+    uint64_t fieldsLoaded = 0;
+    char * filename;
+    tr_benc top;
+    tr_bool boolVal;
+    const tr_bool  wasDirty = tor->isDirty;
 
     filename = getResumeFilename( tor );
 
@@ -637,10 +639,15 @@ loadFromFile( tr_torrent * tor,
 
     if( fieldsToLoad & TR_FR_SPEEDLIMIT )
         fieldsLoaded |= loadSpeedLimits( &top, tor );
-    
+
     if( fieldsToLoad & TR_FR_RATIOLIMIT )
         fieldsLoaded |= loadRatioLimits( &top, tor );
 
+    /* loading the resume file triggers of a lot of changes,
+     * but none of them needs to trigger a re-saving of the
+     * same resume information... */
+    tor->isDirty = wasDirty;
+
     tr_bencFree( &top );
     tr_free( filename );
     return fieldsLoaded;
