@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.c 8771 2009-07-02 02:04:34Z charles $
+ * $Id: torrent.c 8948 2009-08-16 21:09:08Z charles $
  */
 
 #include <sys/types.h> /* stat */
@@ -18,6 +18,7 @@
 #include <assert.h>
 #include <limits.h> /* INT_MAX */
 #include <math.h> /* fabs */
+#include <stdarg.h>
 #include <string.h> /* memcmp */
 #include <stdlib.h> /* qsort */
 
@@ -30,7 +31,7 @@
 #include "completion.h"
 #include "crypto.h" /* for tr_sha1 */
 #include "resume.h"
-#include "fdlimit.h" /* tr_fdFileClose */
+#include "fdlimit.h" /* tr_fdTorrentClose */
 #include "metainfo.h"
 #include "peer-mgr.h"
 #include "platform.h" /* TR_PATH_DELIMITER_STR */
@@ -115,7 +116,8 @@ tr_torrentSetSpeedLimit( tr_torrent * tor, tr_direction dir, int KiB_sec )
     assert( tr_isTorrent( tor ) );
     assert( tr_isDirection( dir ) );
 
-    tr_bandwidthSetDesiredSpeed( tor->bandwidth, dir, KiB_sec );
+    if( tr_bandwidthSetDesiredSpeed( tor->bandwidth, dir, KiB_sec ) )
+        tr_torrentSetDirty( tor );
 }
 
 int
@@ -133,7 +135,8 @@ tr_torrentUseSpeedLimit( tr_torrent * tor, tr_direction dir, tr_bool do_use )
     assert( tr_isTorrent( tor ) );
     assert( tr_isDirection( dir ) );
 
-    tr_bandwidthSetLimited( tor->bandwidth, dir, do_use );
+    if( tr_bandwidthSetLimited( tor->bandwidth, dir, do_use ) )
+        tr_torrentSetDirty( tor );
 }
 
 tr_bool
@@ -148,10 +151,15 @@ tr_torrentUsesSpeedLimit( const tr_torrent * tor, tr_direction dir )
 void
 tr_torrentUseSessionLimits( tr_torrent * tor, tr_bool doUse )
 {
+    tr_bool changed;
+
     assert( tr_isTorrent( tor ) );
 
-    tr_bandwidthHonorParentLimits( tor->bandwidth, TR_UP, doUse );
-    tr_bandwidthHonorParentLimits( tor->bandwidth, TR_DOWN, doUse );
+    changed = tr_bandwidthHonorParentLimits( tor->bandwidth, TR_UP, doUse );
+    changed |= tr_bandwidthHonorParentLimits( tor->bandwidth, TR_DOWN, doUse );
+
+    if( changed )
+        tr_torrentSetDirty( tor );
 }
 
 tr_bool
@@ -172,8 +180,13 @@ tr_torrentSetRatioMode( tr_torrent *  tor, tr_ratiolimit mode )
     assert( tr_isTorrent( tor ) );
     assert( mode==TR_RATIOLIMIT_GLOBAL || mode==TR_RATIOLIMIT_SINGLE || mode==TR_RATIOLIMIT_UNLIMITED  );
 
-    tor->ratioLimitMode = mode;
-    tor->needsSeedRatioCheck = TRUE;
+    if( mode != tor->ratioLimitMode )
+    {
+        tor->ratioLimitMode = mode;
+        tor->needsSeedRatioCheck = TRUE;
+
+        tr_torrentSetDirty( tor );
+    }
 }
 
 tr_ratiolimit
@@ -189,9 +202,14 @@ tr_torrentSetRatioLimit( tr_torrent * tor, double desiredRatio )
 {
     assert( tr_isTorrent( tor ) );
 
-    tor->desiredRatio = desiredRatio;
+    if( (int)(desiredRatio*100.0) != (int)(tor->desiredRatio*100.0) )
+    {
+        tor->desiredRatio = desiredRatio;
 
-    tor->needsSeedRatioCheck = TRUE;
+        tor->needsSeedRatioCheck = TRUE;
+
+        tr_torrentSetDirty( tor );
+    }
 }
 
 double
@@ -252,10 +270,23 @@ tr_torrentGetSeedRatio( const tr_torrent * tor, double * ratio )
 ****
 ***/
 
+void
+tr_torrentSetLocalError( tr_torrent * tor, const char * fmt, ... )
+{
+    va_list ap;
+
+    assert( tr_isTorrent( tor ) );
+
+    va_start( ap, fmt );
+    tor->error = TR_STAT_LOCAL_ERROR;
+    evutil_vsnprintf( tor->errorString, sizeof( tor->errorString ), fmt, ap );
+    va_end( ap );
+}
+
 static void
 onTrackerResponse( void * tracker UNUSED,
-                   void *         vevent,
-                   void *         user_data )
+                   void * vevent,
+                   void * user_data )
 {
     tr_torrent *       tor = user_data;
     tr_tracker_event * event = vevent;
@@ -285,20 +316,18 @@ onTrackerResponse( void * tracker UNUSED,
 
         case TR_TRACKER_WARNING:
             tr_torerr( tor, _( "Tracker warning: \"%s\"" ), event->text );
-            tor->error = -1;
-            tr_strlcpy( tor->errorString, event->text,
-                       sizeof( tor->errorString ) );
+            tor->error = TR_STAT_TRACKER_WARNING;
+            tr_strlcpy( tor->errorString, event->text, sizeof( tor->errorString ) );
             break;
 
         case TR_TRACKER_ERROR:
             tr_torerr( tor, _( "Tracker error: \"%s\"" ), event->text );
-            tor->error = -2;
-            tr_strlcpy( tor->errorString, event->text,
-                       sizeof( tor->errorString ) );
+            tor->error = TR_STAT_TRACKER_ERROR;
+            tr_strlcpy( tor->errorString, event->text, sizeof( tor->errorString ) );
             break;
 
         case TR_TRACKER_ERROR_CLEAR:
-            tor->error = 0;
+            tor->error = TR_STAT_OK;
             tor->errorString[0] = '\0';
             break;
     }
@@ -501,8 +530,7 @@ randomizeTiers( tr_info * info )
     tr_free( r );
 }
 
-static void torrentStart( tr_torrent * tor,
-                          int          reloadProgress );
+static void torrentStart( tr_torrent * tor );
 
 /**
  * Decide on a block size.  constraints:
@@ -604,7 +632,7 @@ torrentRealInit( tr_torrent * tor, const tr_ctor * ctor )
 
     tr_ctorInitTorrentWanted( ctor, tor );
 
-    tor->error   = 0;
+    tor->error = TR_STAT_OK;
 
     tr_bitfieldConstruct( &tor->checkedPieces, tor->info.pieceCount );
     tr_torrentUncheck( tor );
@@ -669,39 +697,39 @@ torrentRealInit( tr_torrent * tor, const tr_ctor * ctor )
     tr_metainfoMigrate( session, &tor->info );
 
     if( doStart )
-        torrentStart( tor, FALSE );
+        torrentStart( tor );
 }
 
-int
-tr_torrentParse( const tr_ctor     * ctor,
-                 tr_info           * setmeInfo )
+tr_parse_result
+tr_torrentParse( const tr_ctor * ctor, tr_info * setmeInfo )
 {
-    int             err = 0;
     int             doFree;
+    tr_bool         didParse;
     tr_info         tmp;
     const tr_benc * metainfo;
     tr_session    * session = tr_ctorGetSession( ctor );
+    tr_parse_result result = TR_PARSE_OK;
 
     if( setmeInfo == NULL )
         setmeInfo = &tmp;
     memset( setmeInfo, 0, sizeof( tr_info ) );
 
-    if( !err && tr_ctorGetMetainfo( ctor, &metainfo ) )
-        return TR_EINVALID;
+    if( tr_ctorGetMetainfo( ctor, &metainfo ) )
+        return TR_PARSE_ERR;
 
-    err = tr_metainfoParse( session, setmeInfo, metainfo );
-    doFree = !err && ( setmeInfo == &tmp );
+    didParse = tr_metainfoParse( session, setmeInfo, metainfo );
+    doFree = didParse && ( setmeInfo == &tmp );
 
-    if( !err && !getBlockSize( setmeInfo->pieceSize ) )
-        err = TR_EINVALID;
+    if( didParse && !getBlockSize( setmeInfo->pieceSize ) )
+        result = TR_PARSE_ERR;
 
-    if( !err && session && tr_torrentExists( session, setmeInfo->hash ) )
-        err = TR_EDUPLICATE;
+    if( didParse && session && tr_torrentExists( session, setmeInfo->hash ) )
+        result = TR_PARSE_DUPLICATE;
 
     if( doFree )
         tr_metainfoFree( setmeInfo );
 
-    return err;
+    return result;
 }
 
 tr_torrent *
@@ -743,7 +771,7 @@ tr_torrentSetDownloadDir( tr_torrent * tor, const char * path )
     {
         tr_free( tor->downloadDir );
         tor->downloadDir = tr_strdup( path );
-        tr_torrentSaveResume( tor );
+        tr_torrentSetDirty( tor );
     }
 }
 
@@ -859,9 +887,8 @@ tr_torrentStat( tr_torrent * tor )
     s = &tor->stats;
     s->id = tor->uniqueId;
     s->activity = tr_torrentGetActivity( tor );
-    s->error  = tor->error;
-    memcpy( s->errorString, tor->errorString,
-           sizeof( s->errorString ) );
+    s->error = tor->error;
+    memcpy( s->errorString, tor->errorString, sizeof( s->errorString ) );
 
     tc = tor->tracker;
     ti = tr_trackerGetAddress( tor->tracker, tor );
@@ -984,7 +1011,7 @@ tr_torrentStat( tr_torrent * tor )
         s->percentRatio = 1.0;
     else if( s->ratio == TR_RATIO_NA )
         s->percentRatio = 0.0;
-    else 
+    else
         s->percentRatio = s->ratio / seedRatio;
 
     tr_torrentUnlock( tor );
@@ -1162,6 +1189,8 @@ tr_torrentResetTransferStats( tr_torrent * tor )
     tor->corruptPrev    += tor->corruptCur;
     tor->corruptCur      = 0;
 
+    tr_torrentSetDirty( tor );
+
     tr_torrentUnlock( tor );
 }
 
@@ -1237,24 +1266,33 @@ freeTorrent( tr_torrent * tor )
 static void
 checkAndStartImpl( void * vtor )
 {
-    time_t now;
     tr_torrent * tor = vtor;
 
     assert( tr_isTorrent( tor ) );
 
     tr_globalLock( tor->session );
 
-    now = time( NULL );
-    tor->isRunning = TRUE;
-    tor->needsSeedRatioCheck = TRUE;
-    *tor->errorString = '\0';
-    tr_torrentResetTransferStats( tor );
-    tor->completeness = tr_cpGetStatus( &tor->completion );
-    tr_torrentSaveResume( tor );
-    tor->startDate = tor->anyDate = now;
-    tr_trackerStart( tor->tracker );
-    tor->dhtAnnounceAt = now + tr_cryptoWeakRandInt( 20 );
-    tr_peerMgrStartTorrent( tor );
+    /** If we had local data before, but it's disappeared,
+        stop the torrent and log an error. */
+    if( tor->preVerifyTotal && !tr_cpHaveTotal( &tor->completion ) )
+    {
+        tr_torrentSetLocalError( tor, _( "Can't find local data.  Try \"Set Location\" to find it, or restart the torrent to re-download." ) );
+        tr_torrentStop( tor );
+    }
+    else
+    {
+        const time_t now = time( NULL );
+        tor->isRunning = TRUE;
+        tor->needsSeedRatioCheck = TRUE;
+        tor->error = TR_STAT_OK;
+        tor->errorString[0] = '\0';
+        tor->completeness = tr_cpGetStatus( &tor->completion );
+        tor->startDate = tor->anyDate = now;
+        tr_torrentResetTransferStats( tor );
+        tr_trackerStart( tor->tracker );
+        tor->dhtAnnounceAt = now + tr_cryptoWeakRandInt( 20 );
+        tr_peerMgrStartTorrent( tor );
+    }
 
     tr_globalUnlock( tor->session );
 }
@@ -1269,7 +1307,7 @@ checkAndStartCB( tr_torrent * tor )
 }
 
 static void
-torrentStart( tr_torrent * tor, int reloadProgress )
+torrentStart( tr_torrent * tor )
 {
     assert( tr_isTorrent( tor ) );
 
@@ -1278,11 +1316,8 @@ torrentStart( tr_torrent * tor, int reloadProgress )
     if( !tor->isRunning )
     {
         tr_verifyRemove( tor );
-
-        if( reloadProgress )
-            tr_torrentLoadResume( tor, TR_FR_PROGRESS, NULL );
-
         tor->isRunning = 1;
+        tor->preVerifyTotal = tr_cpHaveTotal( &tor->completion );
         tr_verifyAdd( tor, checkAndStartCB );
     }
 
@@ -1293,7 +1328,7 @@ void
 tr_torrentStart( tr_torrent * tor )
 {
     if( tr_isTorrent( tor ) )
-        torrentStart( tor, TRUE );
+        torrentStart( tor );
 }
 
 static void
@@ -1304,9 +1339,15 @@ torrentRecheckDoneImpl( void * vtor )
     assert( tr_isTorrent( tor ) );
     tr_torrentRecheckCompleteness( tor );
 
-    if( tor->startAfterVerify )
+    if( tor->preVerifyTotal && !tr_cpHaveTotal( &tor->completion ) )
+    {
+        tr_torrentSetLocalError( tor, _( "Can't find local data.  Try \"Set Location\" to find it, or restart the torrent to re-download." ) );
+        tr_torrentStop( tor );
+    }
+    else if( tor->startAfterVerify )
     {
         tor->startAfterVerify = FALSE;
+
         tr_torrentStart( tor );
     }
 }
@@ -1319,9 +1360,11 @@ torrentRecheckDoneCB( tr_torrent * tor )
     tr_runInEventThread( tor->session, torrentRecheckDoneImpl, tor );
 }
 
-void
-tr_torrentVerify( tr_torrent * tor )
+static void
+verifyTorrent( void * vtor )
 {
+    tr_torrent * tor = vtor;
+
     assert( tr_isTorrent( tor ) );
     tr_globalLock( tor->session );
 
@@ -1329,18 +1372,37 @@ tr_torrentVerify( tr_torrent * tor )
     tr_verifyRemove( tor );
 
     /* if the torrent's running, stop it & set the restart-after-verify flag */
-    if( tor->isRunning ) {
+    if( tor->startAfterVerify || tor->isRunning ) {
         tr_torrentStop( tor );
         tor->startAfterVerify = TRUE;
     }
 
     /* add the torrent to the recheck queue */
+    tor->preVerifyTotal = tr_cpHaveTotal( &tor->completion );
     tr_torrentUncheck( tor );
     tr_verifyAdd( tor, torrentRecheckDoneCB );
 
     tr_globalUnlock( tor->session );
 }
 
+void
+tr_torrentVerify( tr_torrent * tor )
+{
+    if( tr_isTorrent( tor ) )
+        tr_runInEventThread( tor->session, verifyTorrent, tor );
+}
+
+void
+tr_torrentSave( tr_torrent * tor )
+{
+    assert( tr_isTorrent( tor ) );
+
+    if( tor->isDirty ) {
+        tor->isDirty = FALSE;
+        tr_torrentSaveResume( tor );
+    }
+}
+
 static void
 stopTorrent( void * vtor )
 {
@@ -1353,6 +1415,9 @@ stopTorrent( void * vtor )
     tr_trackerStop( tor->tracker );
 
     tr_fdTorrentClose( tor->uniqueId );
+
+    if( !tor->isDeleting )
+        tr_torrentSave( tor );
 }
 
 void
@@ -1365,8 +1430,6 @@ tr_torrentStop( tr_torrent * tor )
         tr_globalLock( tor->session );
 
         tor->isRunning = 0;
-        if( !tor->isDeleting )
-            tr_torrentSaveResume( tor );
         tr_runInEventThread( tor->session, stopTorrent, tor );
 
         tr_globalUnlock( tor->session );
@@ -1385,14 +1448,15 @@ closeTorrent( void * vtor )
     tr_bencDictAddInt( d, "id", tor->uniqueId );
     tr_bencDictAddInt( d, "date", time( NULL ) );
 
-    tr_torrentSaveResume( tor );
-    tor->isRunning = 0;
     stopTorrent( tor );
+
     if( tor->isDeleting )
     {
         tr_metainfoRemoveSaved( tor->session, &tor->info );
         tr_torrentRemoveResume( tor );
     }
+
+    tor->isRunning = 0;
     freeTorrent( tor );
 }
 
@@ -1527,7 +1591,7 @@ tr_torrentRecheckCompleteness( tr_torrent * tor )
             tor->doneDate = tor->anyDate = time( NULL );
         }
 
-        tr_torrentSaveResume( tor );
+        tr_torrentSetDirty( tor );
     }
 
     tr_torrentUnlock( tor );
@@ -1570,7 +1634,7 @@ tr_torrentSetFilePriorities( tr_torrent *      tor,
     for( i = 0; i < fileCount; ++i )
         tr_torrentInitFilePriority( tor, files[i], priority );
 
-    tr_torrentSaveResume( tor );
+    tr_torrentSetDirty( tor );
     tr_torrentUnlock( tor );
 }
 
@@ -1714,7 +1778,7 @@ tr_torrentSetFileDLs( tr_torrent *      tor,
 
     tr_torrentLock( tor );
     tr_torrentInitFileDLs( tor, files, fileCount, doDownload );
-    tr_torrentSaveResume( tor );
+    tr_torrentSetDirty( tor );
     tr_torrentUnlock( tor );
 }
 
@@ -1736,7 +1800,12 @@ tr_torrentSetPriority( tr_torrent * tor, tr_priority_t priority )
     assert( tr_isTorrent( tor ) );
     assert( tr_isPriority( priority ) );
 
-    tor->bandwidth->priority = priority;
+    if( tor->bandwidth->priority != priority )
+    {
+        tor->bandwidth->priority = priority;
+
+        tr_torrentSetDirty( tor );
+    }
 }
 
 /***
@@ -1800,10 +1869,11 @@ tr_torrentReqIsValid( const tr_torrent * tor,
     else if( tr_pieceOffset( tor, index, offset, length ) > tor->info.totalSize )
         err = 5;
 
-    if( err ) fprintf( stderr, "index %lu offset %lu length %lu err %d\n",
-                       (unsigned long)index, (unsigned long)offset,
-                       (unsigned long)length,
-                       err );
+    if( err ) tr_tordbg( tor, "index %lu offset %lu length %lu err %d\n",
+                              (unsigned long)index,
+                              (unsigned long)offset,
+                              (unsigned long)length,
+                              err );
 
     return !err;
 }
@@ -1962,7 +2032,7 @@ tr_torrentSetAnnounceList( tr_torrent *            tor,
 
         /* try to parse it back again, to make sure it's good */
         memset( &tmpInfo, 0, sizeof( tr_info ) );
-        if( !tr_metainfoParse( tor->session, &tmpInfo, &metainfo ) )
+        if( tr_metainfoParse( tor->session, &tmpInfo, &metainfo ) )
         {
             /* it's good, so keep these new trackers and free the old ones */
 
@@ -1998,8 +2068,7 @@ tr_torrentSetAddedDate( tr_torrent * tor,
 }
 
 void
-tr_torrentSetActivityDate( tr_torrent * tor,
-                           time_t       t )
+tr_torrentSetActivityDate( tr_torrent * tor, time_t t )
 {
     assert( tr_isTorrent( tor ) );
 
@@ -2267,7 +2336,7 @@ setLocation( void * vdata )
             const tr_file * f = &tor->info.files[i];
             char * oldpath = tr_buildPath( tor->downloadDir, f->name, NULL );
             char * newpath = tr_buildPath( location, f->name, NULL );
-            
+
             if( do_move )
             {
                 errno = 0;
