@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: Torrent.m 8800 2009-07-14 00:11:50Z livings124 $
+ * $Id: Torrent.m 8873 2009-08-05 02:13:45Z livings124 $
  *
  * Copyright (c) 2006-2009 Transmission authors and contributors
  *
@@ -31,7 +31,6 @@
 @interface Torrent (Private)
 
 - (id) initWithPath: (NSString *) path hash: (NSString *) hashString torrentStruct: (tr_torrent *) torrentStruct lib: (tr_session *) lib
-        publicTorrent: (NSNumber *) publicTorrent publicTorrentLocation: (NSString *) publicTorrentLoc
         downloadFolder: (NSString *) downloadFolder
         useIncompleteFolder: (NSNumber *) useIncompleteFolder incompleteFolder: (NSString *) incompleteFolder
         waitToStart: (NSNumber *) waitToStart
@@ -55,8 +54,6 @@
 
 - (void) updateAllTrackers: (NSMutableArray *) trackers;
 
-+ (void) trashFile: (NSString *) path;
-
 - (void) setTimeMachineExclude: (BOOL) exclude forPath: (NSString *) path;
 
 @end
@@ -80,28 +77,18 @@ int trashDataFile(const char * filename)
 
 @implementation Torrent
 
-- (id) initWithPath: (NSString *) path location: (NSString *) location deleteTorrentFile: (torrentFileState) torrentDelete
+- (id) initWithPath: (NSString *) path location: (NSString *) location deleteTorrentFile: (BOOL) torrentDelete
         lib: (tr_session *) lib
 {
     self = [self initWithPath: path hash: nil torrentStruct: NULL lib: lib
-            publicTorrent: torrentDelete != TORRENT_FILE_DEFAULT ? [NSNumber numberWithBool: torrentDelete == TORRENT_FILE_SAVE] : nil
-            publicTorrentLocation: path
             downloadFolder: location
             useIncompleteFolder: nil incompleteFolder: nil
             waitToStart: nil groupValue: nil addedTrackers: nil];
     
     if (self)
     {
-        //if the public and private torrent files are the same, then there is no public torrent
-        if ([[self torrentLocation] isEqualToString: path])
-        {
-            fPublicTorrent = NO;
-            [fPublicTorrentLocation release];
-            fPublicTorrentLocation = nil;
-        }
-        else if (!fPublicTorrent)
+        if (torrentDelete && ![[self torrentLocation] isEqualToString: path])
             [Torrent trashFile: path];
-        else;
     }
     return self;
 }
@@ -109,7 +96,6 @@ int trashDataFile(const char * filename)
 - (id) initWithTorrentStruct: (tr_torrent *) torrentStruct location: (NSString *) location lib: (tr_session *) lib
 {
     self = [self initWithPath: nil hash: nil torrentStruct: torrentStruct lib: lib
-            publicTorrent: [NSNumber numberWithBool: NO] publicTorrentLocation: nil
             downloadFolder: location
             useIncompleteFolder: nil incompleteFolder: nil
             waitToStart: nil groupValue: nil addedTrackers: nil];
@@ -122,8 +108,6 @@ int trashDataFile(const char * filename)
     self = [self initWithPath: [history objectForKey: @"InternalTorrentPath"]
                 hash: [history objectForKey: @"TorrentHash"]
                 torrentStruct: NULL lib: lib
-                publicTorrent: [history objectForKey: @"PublicCopy"]
-                publicTorrentLocation: [history objectForKey: @"TorrentPath"]
                 downloadFolder: [history objectForKey: @"DownloadFolder"]
                 useIncompleteFolder: [history objectForKey: @"UseIncompleteFolder"]
                 incompleteFolder: [history objectForKey: @"IncompleteFolder"]
@@ -173,7 +157,6 @@ int trashDataFile(const char * filename)
     NSMutableDictionary * history = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                     [self torrentLocation], @"InternalTorrentPath",
                     [self hashString], @"TorrentHash",
-                    [NSNumber numberWithBool: fPublicTorrent], @"PublicCopy",
                     fDownloadFolder, @"DownloadFolder",
                     [NSNumber numberWithBool: fUseIncompleteFolder], @"UseIncompleteFolder",
                     [NSNumber numberWithBool: [self isActive]], @"Active",
@@ -183,9 +166,6 @@ int trashDataFile(const char * filename)
     
     if (fIncompleteFolder)
         [history setObject: fIncompleteFolder forKey: @"IncompleteFolder"];
-
-    if (fPublicTorrent)
-        [history setObject: [self publicTorrentLocation] forKey: @"TorrentPath"];
 	
     return history;
 }
@@ -206,8 +186,6 @@ int trashDataFile(const char * filename)
     [fDownloadFolder release];
     [fIncompleteFolder release];
     
-    [fPublicTorrentLocation release];
-    
     [fIcon release];
     
     [fFileList release];
@@ -463,10 +441,19 @@ int trashDataFile(const char * filename)
     [[NSWorkspace sharedWorkspace] selectFile: [self dataLocation] inFileViewerRootedAtPath: nil];
 }
 
-- (void) revealPublicTorrent
+#warning should be somewhere else?
++ (void) trashFile: (NSString *) path
 {
-    if (fPublicTorrent)
-        [[NSWorkspace sharedWorkspace] selectFile: fPublicTorrentLocation inFileViewerRootedAtPath: nil];
+    //attempt to move to trash
+    if (![[NSWorkspace sharedWorkspace] performFileOperation: NSWorkspaceRecycleOperation
+        source: [path stringByDeletingLastPathComponent] destination: @""
+        files: [NSArray arrayWithObject: [path lastPathComponent]] tag: nil])
+    {
+        //if cannot trash, just delete it (will work if it's on a remote volume)
+        NSError * error;
+        if (![[NSFileManager defaultManager] removeItemAtPath: path error: &error])
+            NSLog(@"Could not trash %@: %@", path, [error localizedDescription]);
+    }
 }
 
 - (void) trashData
@@ -474,18 +461,6 @@ int trashDataFile(const char * filename)
     tr_torrentDeleteLocalData(fHandle, trashDataFile);
 }
 
-- (void) trashTorrent
-{
-    if (fPublicTorrent)
-    {
-        [Torrent trashFile: fPublicTorrentLocation];
-        [fPublicTorrentLocation release];
-        fPublicTorrentLocation = nil;
-        
-        fPublicTorrent = NO;
-    }
-}
-
 - (void) moveTorrentDataFileTo: (NSString *) folder
 {
     NSString * oldFolder = [self downloadFolder];
@@ -878,21 +853,11 @@ int trashDataFile(const char * filename)
     return [NSString stringWithUTF8String: fInfo->torrent];
 }
 
-- (NSString *) publicTorrentLocation
-{
-    return fPublicTorrentLocation;
-}
-
 - (NSString *) dataLocation
 {
     return [[self downloadFolder] stringByAppendingPathComponent: [self name]];
 }
 
-- (BOOL) publicTorrent
-{
-    return fPublicTorrent;
-}
-
 - (CGFloat) progress
 {
     return fStat->percentComplete;
@@ -955,12 +920,17 @@ int trashDataFile(const char * filename)
 
 - (BOOL) isError
 {
-    return fStat->error != TR_OK;
+    return fStat->error == TR_STAT_LOCAL_ERROR || fStat->error == TR_STAT_TRACKER_ERROR;
+}
+
+- (BOOL) isErrorOrWarning
+{
+    return fStat->error != TR_STAT_OK;
 }
 
 - (NSString *) errorMessage
 {
-    if (![self isError])
+    if (![self isErrorOrWarning])
         return @"";
     
     NSString * error;
@@ -1093,9 +1063,16 @@ int trashDataFile(const char * filename)
 {
     NSString * string;
     
-    if ([self isError])
+    if ([self isErrorOrWarning])
     {
-        string = NSLocalizedString(@"Error", "Torrent -> status string");
+        switch (fStat->error)
+        {
+            case TR_STAT_LOCAL_ERROR: string = NSLocalizedString(@"Error", "Torrent -> status string"); break;
+            case TR_STAT_TRACKER_ERROR: string = NSLocalizedString(@"Tracker returned an error", "Torrent -> status string"); break;
+            case TR_STAT_TRACKER_WARNING: string = NSLocalizedString(@"Tracker returned a warning", "Torrent -> status string"); break;
+            default: NSAssert(NO, @"unknown error state");
+        }
+        
         NSString * errorString = [self errorMessage];
         if (errorString && ![errorString isEqualToString: @""])
             string = [string stringByAppendingFormat: @": %@", errorString];
@@ -1595,7 +1572,6 @@ int trashDataFile(const char * filename)
 @implementation Torrent (Private)
 
 - (id) initWithPath: (NSString *) path hash: (NSString *) hashString torrentStruct: (tr_torrent *) torrentStruct lib: (tr_session *) lib
-        publicTorrent: (NSNumber *) publicTorrent publicTorrentLocation: (NSString *) publicTorrentLoc
         downloadFolder: (NSString *) downloadFolder
         useIncompleteFolder: (NSNumber *) useIncompleteFolder incompleteFolder: (NSString *) incompleteFolder
         waitToStart: (NSNumber *) waitToStart
@@ -1605,10 +1581,6 @@ int trashDataFile(const char * filename)
         return nil;
     
     fDefaults = [NSUserDefaults standardUserDefaults];
-
-    fPublicTorrent = publicTorrentLoc && (publicTorrent ? [publicTorrent boolValue] : ![fDefaults boolForKey: @"DeleteOriginalTorrent"]);
-    if (fPublicTorrent)
-        fPublicTorrentLocation = [publicTorrentLoc retain];
     
     fDownloadFolder = downloadFolder ? downloadFolder : [fDefaults stringForKey: @"DownloadFolder"];
     fDownloadFolder = [[fDownloadFolder stringByExpandingTildeInPath] retain];
@@ -1637,20 +1609,20 @@ int trashDataFile(const char * filename)
         tr_ctorSetPaused(ctor, TR_FORCE, YES);
         tr_ctorSetPeerLimit(ctor, TR_FALLBACK, [fDefaults integerForKey: @"PeersTorrent"]);
         
-        int result = TR_EINVALID;
+        int result = TR_PARSE_ERR;
         if (path)
             result = tr_ctorSetMetainfoFromFile(ctor, [path UTF8String]);
         
         //backup - shouldn't be needed after upgrade to 1.70
-        if (result != TR_OK && hashString)
+        if (result != TR_PARSE_OK && hashString)
             result = tr_ctorSetMetainfoFromHash(ctor, [hashString UTF8String]);
         
-        if (result == TR_OK)
+        if (result == TR_PARSE_OK)
         {
             tr_info info;
             result = tr_torrentParse(ctor, &info);
             
-            if (result == TR_OK)
+            if (result == TR_PARSE_OK)
             {
                 NSString * currentDownloadFolder = [self shouldUseIncompleteFolderForName: [NSString stringWithUTF8String: info.name]]
                                                     ? fIncompleteFolder : fDownloadFolder;
@@ -1658,7 +1630,7 @@ int trashDataFile(const char * filename)
                 
                 fHandle = tr_torrentNew(ctor, NULL);
             }
-            if (result != TR_EINVALID)
+            if (result != TR_PARSE_ERR)
                 tr_metainfoFree(&info);
         }
         
@@ -1952,20 +1924,6 @@ int trashDataFile(const char * filename)
     tr_free(trackerStructs);
 }
 
-+ (void) trashFile: (NSString *) path
-{
-    //attempt to move to trash
-    if (![[NSWorkspace sharedWorkspace] performFileOperation: NSWorkspaceRecycleOperation
-        source: [path stringByDeletingLastPathComponent] destination: @""
-        files: [NSArray arrayWithObject: [path lastPathComponent]] tag: nil])
-    {
-        //if cannot trash, just delete it (will work if it's on a remote volume)
-        NSError * error;
-        if (![[NSFileManager defaultManager] removeItemAtPath: path error: &error])
-            NSLog(@"Could not trash %@: %@", path, [error localizedDescription]);
-    }
-}
-
 - (void) setTimeMachineExclude: (BOOL) exclude forPath: (NSString *) path
 {
     CSBackupSetItemExcluded((CFURLRef)[NSURL fileURLWithPath: path], exclude, true);
