@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: mainwin.cc 8769 2009-07-01 23:40:07Z charles $
+ * $Id: mainwin.cc 8890 2009-08-11 18:57:49Z charles $
  */
 
 #include <cassert>
@@ -227,7 +227,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     myTrayIcon.setIcon( QApplication::windowIcon( ) );
 
     connect( &myPrefs, SIGNAL(changed(int)), this, SLOT(refreshPref(int)) );
-    connect( ui.action_ShowMainWindow, SIGNAL(toggled(bool)), this, SLOT(toggleWindows()));
+    connect( ui.action_ShowMainWindow, SIGNAL(toggled(bool)), this, SLOT(toggleWindows(bool)));
     connect( &myTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
              this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
 
@@ -883,16 +883,31 @@ TrMainWindow :: setStatusbarVisible( bool visible )
 **/
 
 void
-TrMainWindow :: toggleWindows( )
+TrMainWindow :: toggleWindows( bool doShow )
 {
-    setVisible( !isVisible( ) );
+    if( !doShow )
+    {
+        hide( );
+    }
+    else
+    {
+        if ( !isVisible( ) ) show( );
+        if ( isMinimized( ) ) showNormal( );
+        activateWindow( );
+        raise( );
+    }
 }
 
 void
 TrMainWindow :: trayActivated( QSystemTrayIcon::ActivationReason reason )
 {
     if( reason == QSystemTrayIcon::Trigger )
-        ui.action_ShowMainWindow->toggle( );
+    {
+        if( isMinimized ( ) )
+            toggleWindows( true );
+        else
+            ui.action_ShowMainWindow->toggle( );
+    }
 }
 
 
